
#include <jni.h>

#define PATH_SEPARATOR ';' /* define it to be ':' on Solaris */
#define USER_CLASSPATH "." /* where Prog.class is */

int main() {
    JNIEnv *env;
    JavaVM *jvm;
    jint res, fval;
    jclass cls, fcls, lcls;
    jmethodID mid, fcon, lcon;
    jfieldID fid;
    jstring jstr;
    jclass stringClass;
    jobjectArray args;
	jobject frame, panel, label;
#ifdef JNI_VERSION_1_2
    JavaVMInitArgs vm_args;
    JavaVMOption options[1];
    options[0].optionString =
        "-Djava.class.path=" USER_CLASSPATH;
    vm_args.version = 0x00010002;
    vm_args.options = options;
    vm_args.nOptions = 1;
    vm_args.ignoreUnrecognized = JNI_TRUE;
/* Create the Java VM */
    res = JNI_CreateJavaVM(&jvm, (void**)&env, &vm_args);
#else
    JDK1_1InitArgs vm_args;
    char classpath[1024];
    vm_args.version = 0x00010001;
    JNI_GetDefaultJavaVMInitArgs(&vm_args);
    /* Append USER_CLASSPATH to the default system class path */
    sprintf(classpath, "%s%c%s",
            vm_args.classpath, PATH_SEPARATOR, USER_CLASSPATH);
    vm_args.classpath = classpath;
/* Create the Java VM */
    res = JNI_CreateJavaVM(&jvm, &env, &vm_args);
#endif /* JNI_VERSION_1_2 */

if (res < 0) {
        fprintf(stderr, "Can't create Java VM\n");
        exit(1);
    }
    cls = (*env)->FindClass(env, "trybut");
    if (cls == 0) {
        goto destroy;
    }
    fcls = (*env)->FindClass(env, "javax/swing/JFrame");
    if (fcls == 0) {
        goto destroy;
    }
    lcls = (*env)->FindClass(env, "javax/swing/JLabel");
    if (lcls == 0) {
        goto destroy;
    }
#define CHECK(x) if (x == 0) { goto destroy; }
 	fcon = (*env)->GetMethodID( env, fcls, "<init>", "()V");
CHECK(fcon)
	frame = (*env)->NewObject( env, fcls, fcon, "HelloWorldSwing" );
CHECK(frame)
 	lcon = (*env)->GetMethodID( env, lcls, "<init>", "()V");
CHECK(lcon)
	label = (*env)->NewObject( env, lcls, lcon, "Hello World" );
CHECK(label)

 	fcon = (*env)->GetMethodID( env, fcls, "setDefaultCloseOperation", "(I)V");
CHECK(fcon)
    fid = (*env)->GetStaticFieldID(env, fcls, "EXIT_ON_CLOSE",
                                    "I");
CHECK(fid)
    fval = (*env)->GetStaticIntField(env, fcls, fid);
CHECK(fval)
    (*env)->CallNonvirtualVoidMethod(env, frame, fcls, fcon, fval ); 
/* at this point frame setDefaultCloseOperation is done */

  fcon = (*env)->GetMethodID( env, fcls, "getContentPane", 
   "()Ljava/awt/Container;");
CHECK(fcon)
    (*env)->CallNonvirtualVoidMethod(env, frame, fcls, fcon, fval ); 
/* getContentPane is done, yielding a container */
 	fcon = (*env)->GetMethodID( env, fcls, "add", 
	  "(Ljava/awt/Component;)Ljava/awt/Component;");
CHECK(fcon)
    (*env)->CallNonvirtualVoidMethod(env, frame, fcls, fcon, label ); 
/* now label is installed */
    
 
/*
    mid = (*env)->GetStaticMethodID(env, cls, "main",
                                    "([Ljava/lang/String;)V");
    fid = (*env)->GetStaticFieldID(env, cls, "demoSFieldA",
                                    "[I");
    fval = (*env)->GetStaticIntField(env, cls, fid);
    printf("FIELD VALU = %d\n", (int)fval);
    if (mid == 0) {
        goto destroy;
    }
    jstr = (*env)->NewStringUTF(env, " from C!"); 
    if (jstr == 0) {
        goto destroy;
    }
    stringClass = (*env)->FindClass(env, "java/lang/String");
    args = (*env)->NewObjectArray(env, 1, stringClass, jstr);
    if (args == 0) {
        goto destroy;
    }
    (*env)->CallStaticVoidMethod(env, cls, mid, args);
*/

destroy:
    if ((*env)->ExceptionOccurred(env)) {
        (*env)->ExceptionDescribe(env);
    }
    (*jvm)->DestroyJavaVM(jvm);
    return 0;
}
